package com.qliktrialreactnativestraighttable;

import android.graphics.Bitmap;
import android.graphics.Color;
import android.text.Html;
import android.text.Spanned;
import android.text.TextUtils;
import android.text.method.LinkMovementMethod;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;

import androidx.core.text.HtmlCompat;
import androidx.recyclerview.widget.RecyclerView;

public class RowViewHolder extends RecyclerView.ViewHolder  {
  private int startIndex, numColumns;
  private final LinearLayout row;
  private final DataProvider dataProvider;
  private boolean firstColumnOnly = false;

  public RowViewHolder(View view, DataProvider dp, boolean firstColumnOnly) {
    super(view);
    this.firstColumnOnly = firstColumnOnly;
    row = (LinearLayout) view;
    dataProvider = dp;
  }

  public void setBackGroundColor(int color) {
    row.setBackgroundColor(color);
  }
  public void setData(DataRow dataRow, int rowHeight, CellContentStyle cellContentStyle) {
    numColumns = dataProvider.dataColumns.size();
    if(firstColumnOnly) {
      numColumns = 1;
    }
    int numCells = row.getChildCount();
    int extraCells = row.getChildCount() - numColumns;
    if(extraCells > 0) {
      row.removeViews(numCells - extraCells, extraCells);
    }
    if(dataRow.cells.size() < numColumns) {
      return;
    }
    for(int i = 0; i < numColumns; i++) {
      DataCell cell = dataRow.cells.get(i);
      DataColumn column = DataProvider.getDataColumnByIdx(cell.colIdx, dataProvider.dataColumns);
      int columnIndex = cell.rawColIdx;

      if(columnIndex >= row.getChildCount()) {
        TableView tableView = dataProvider.tableView;
        CellView cellView = new CellView(tableView.getContext(), column.representation.type, tableView.selectionsEngine, tableView, tableView.isFirstColumnFrozen, column);
        row.addView(cellView);
      }

      CellView cellView = (CellView) row.getChildAt(columnIndex);
      if(column.representation.type.equals("image") && !dataProvider.isDataView) {
        LinearLayout.LayoutParams layout = new LinearLayout.LayoutParams(column.width, ViewGroup.LayoutParams.MATCH_PARENT);
        cellView.setLayoutParams(layout);
        cellView.convertCellContentType("image", column);
        cellView.setData(cell, dataRow, column);

        Bitmap imageBitmap = dataProvider.tableView.imageLoader.getImageData(cell.imageUrl);
        if(imageBitmap == null) {
          continue;
        }
        ClickableImageView imageView = (ClickableImageView) cellView.content;
        imageView.setImageBitmap(imageBitmap);
        imageView.scaleAndPositionImage(column, imageBitmap);
      } else if(column.representation.type.equals("miniChart") && !dataProvider.isDataView) {
        LinearLayout.LayoutParams cellViewLayoutParams = new LinearLayout.LayoutParams(column.width, ViewGroup.LayoutParams.MATCH_PARENT);
        cellView.setLayoutParams(cellViewLayoutParams);
        cellView.setData(cell, dataRow, column);
        cellView.convertCellContentType("miniChart", column);
        MiniChartView miniChartView = (MiniChartView) cellView.content;
        miniChartView.setData(cell, column);
      } else {
        cellView.convertCellContentType("text", column);
        ClickableTextView textView = (ClickableTextView) cellView.content;
        RelativeLayout.LayoutParams textViewLayoutParams = new RelativeLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        textView.setLayoutParams(textViewLayoutParams);
        LinearLayout.LayoutParams cellViewLayoutParams = new LinearLayout.LayoutParams(column.width, ViewGroup.LayoutParams.MATCH_PARENT);
        cellViewLayoutParams.gravity = Gravity.TOP;
        cellView.setLayoutParams(cellViewLayoutParams);

        if(column.representation.type.equals("text") || dataProvider.isDataView) {
          cell.indicator = null;
        }

        cellView.setData(cell, dataRow, column);
        if(column.representation.type.equals("url") && !dataProvider.isDataView) {
          setupHyperLink(textView);
        }
        if(column.isDim && cellContentStyle.wrap) {
          textView.setMaxLines(rowHeight/cellContentStyle.lineHeight);
        } else {
          textView.setMaxLines(1);
        }
        textView.setGravity(column.textAlignment | Gravity.CENTER_VERTICAL);
      }
    }
  }

  private void setupHyperLink(ClickableTextView textView) {
    String htmlText = String.format("<a href=\"%s\">%s</a>", textView.linkUrl, textView.linkLabel);
    Spanned result = HtmlCompat.fromHtml(htmlText, HtmlCompat.FROM_HTML_MODE_LEGACY);
    textView.setText(result);
    textView.setMovementMethod(LinkMovementMethod.getInstance());
    textView.setEllipsize(TextUtils.TruncateAt.END);
    textView.setMaxLines(1);
    // this must be set for elipse to show.  Weird but true.
    // https://stackoverflow.com/questions/1141651/how-to-set-a-long-string-in-a-text-view-in-a-single-line-with-horizontal-scroll
    textView.setHorizontallyScrolling(true);
  }

  public void onRecycled() {
    for(int i = 0; i <  row.getChildCount(); i++) {
      View view = row.getChildAt(i);
      if(view instanceof RelativeLayout) {
        return;
      }
      SelectionsObserver observerView = (SelectionsObserver) row.getChildAt(i);
      observerView.onRecycled();
    }
  }

  public boolean updateWidth(float deltaWidth, int column) {
    if(column > numColumns - 1) {
      return true;
    }
    View view = row.getChildAt(column);
    int currentWidth = dataProvider.dataColumns.get(column).width;
    float newWidth = currentWidth + deltaWidth;

    if(newWidth < dataProvider.minWidth) {
      return false;
    }

    if (!updateNeighbour(deltaWidth, column)) {
      return false;
    }
    ViewGroup.LayoutParams params = view.getLayoutParams();
    params.width = (int) newWidth;
    view.setLayoutParams(params);

    DataColumn dataColumn = dataProvider.dataColumns.get(column);
    checkTextWrap(dataColumn);
    checkNeighbourTextWrap(column);

    return true;
  }

  private void checkNeighbourTextWrap(int column) {
    if (column + 1 < numColumns ) {
      DataColumn dataColumn = dataProvider.dataColumns.get(column);
      checkTextWrap(dataColumn);
    }
  }

  private void checkTextWrap(DataColumn dataColumn) {
    if(dataColumn.isDim && dataColumn.isText() ) {
      CellView cellView = (CellView) row.getChildAt(dataColumn.columnIndex);
      ClickableTextView textView = (ClickableTextView)cellView.content;
      textView.testTextWrap(dataColumn);
    }
  }


  public boolean setWidth(int width, int column) {
    if(column > numColumns - 1) {
      return true;
    }

    View view = row.getChildAt(column);
    ViewGroup.LayoutParams params = view.getLayoutParams();
    params.width = width;
    view.setLayoutParams(params);

    return true;
  }

  private boolean updateNeighbour(float deltaWidth, int column) {
    if (column + 1 < numColumns ) {
      View neighbour =  row.getChildAt(column + 1);
      int currentWidth = dataProvider.dataColumns.get(column + 1).width;
      float newWidth = currentWidth - deltaWidth;
      if (newWidth < dataProvider.minWidth) {
        return false;
      }
      ViewGroup.LayoutParams params = neighbour.getLayoutParams();
      params.width = (int)newWidth;
      neighbour.setLayoutParams(params);
    }
    return true;
  }

  public int getLineCount(DataColumn column) {
    CellView cellView = (CellView) row.getChildAt(column.columnIndex);
    ClickableTextView textView = (ClickableTextView) cellView.content;
    return textView.getMeasuredLineCount();
  }

  public void updateHeight(int rowHeight) {
    ViewGroup.LayoutParams params = row.getLayoutParams();
    params.height = rowHeight;
    row.setLayoutParams(params);

  }

  public int initialMeasure() {
    int maxLines = 0;
    for (DataColumn column: dataProvider.dataColumns) {
      if(column.isText() && column.isDim) {
        CellView cellView = (CellView) row.getChildAt(column.columnIndex);
        if(cellView != null) {
          ClickableTextView clickableTextView = (ClickableTextView) cellView.content;
          maxLines = Math.max(clickableTextView.measureLines(column), maxLines);
        }
      }
    }
    return  maxLines;
  }

  public void initializeHeight(int rowHeight, CellContentStyle cellContentStyle) {
    int lines = rowHeight / cellContentStyle.lineHeight;

    for (DataColumn column: dataProvider.dataColumns) {
      if(column.isText() && column.isDim) {
        if(column.columnIndex < row.getChildCount()) {
          CellView cellView = (CellView) row.getChildAt(column.columnIndex);
          ClickableTextView clickableTextView = (ClickableTextView) cellView.content;
          clickableTextView.setMaxLines(lines);
          clickableTextView.setGravity(column.textAlignment | Gravity.CENTER_VERTICAL);
          clickableTextView.requestLayout();
        }
      }
    }

    ViewGroup.LayoutParams params = row.getLayoutParams();
    params.height = rowHeight;
    row.setLayoutParams(params);
    row.requestLayout();

  }
}
